unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, LCLIntf, StdCtrls,
  ExtCtrls, Spin;

type

  TSliderDirection = (sdRight, sdLeft);

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    CheckBox1: TCheckBox;
    SpinEdit1: TSpinEdit;
    Timer1: TTimer;
    procedure Button1Click({%H-}Sender: TObject);
    procedure CheckBox1Change(Sender: TObject);
    procedure FormCreate({%H-}Sender: TObject);
    procedure FormDestroy({%H-}Sender: TObject);
    procedure FormPaint({%H-}Sender: TObject);
    procedure SpinEdit1Change(Sender: TObject);
    procedure Timer1Timer({%H-}Sender: TObject);
  private
    FImages: Array[0..3] of TCustomBitmap;
    FRadius: Integer;
    FMargin: Integer;
    FSliderRange: Integer;
    FSliderPos: Integer;
    FSliderDirection: TSliderDirection;
    FRoll: Boolean;
  public
    property Roll: Boolean read FRoll write FRoll;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}
{$R images.res}

uses
  Math, FPImage, IntfGraphics;

function RotatePoint(const APoint: TPoint; AAngle: Double): TPoint;
var
  sa, ca: Double;
begin
  SinCos(AAngle, sa, ca);
  Result.X := Round( ca * APoint.X + sa * APoint.Y);
  Result.Y := Round(-sa * APoint.X + ca * APoint.Y);
end;

procedure RotateImage(Img: TFPCustomImage; Angle: Double);
var
  Buffer: TFPCustomImage;
  x, y: Integer;
  C, P: TPoint;
begin
  Buffer := TFPMemoryImage.Create(Img.Width, Img.Height);
  C := Point(Img.Width div 2, Img.Height div 2);
  for y := 0 to Buffer.Height-1 do
    for x := 0 to Buffer.Width-1 do
    begin
      P := RotatePoint(Point(x, y) - C, Angle) + C;
      if (P.X >= 0) and (P.Y >= 0) and (P.X < Img.Width) and (P.Y < Img.Height) then
        Buffer.Colors[x, y] := Img.Colors[P.X, P.Y]
      else
        Buffer.Colors[x, y] := colTransparent;
    end;
  for y := 0 to Img.Height-1 do
    for x := 0 to Img.Width-1 do
      Img.Colors[x, y] := Buffer[x, y];
  Buffer.Free;
end;

procedure BlendImages(Img1, Img2: TFPCustomImage; AFactor: Double);
var
  x, y: Integer;
  r,g,b,a: Word;
  f1, f2: Double;
begin
  f1 := 1.0 - AFactor;
  f2 := AFactor;
  for y := 0 to Img1.Height-1 do
    for x := 0 to Img1.Width-1 do
    begin
      r := round(f1 * Img1.Colors[x, y].Red   + f2 * Img2.Colors[x, y].Red);
      g := round(f1 * Img1.Colors[x, y].Green + f2 * Img2.Colors[x, y].Green);
      b := round(f1 * Img1.Colors[x, y].Blue  + f2 * Img2.Colors[x, y].Blue);
      a := round(f1 * Img1.Colors[x, y].Alpha + f2 * Img2.Colors[x, y].Alpha);
      Img1.Colors[x, y] := FPColor(r, g, b, a);
    end;
end;


{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var
  i : integer;
begin
  for i := 0 to High(FImages) do
    FImages[i] := TPortableNetworkGraphic.Create;
  FImages[0].LoadFromResourceName(HInstance,'BACKGROUND_RED');
  FImages[1].LoadFromResourceName(HInstance,'BACKGROUND_GREEN');
  FImages[2].LoadFromResourceName(HInstance,'EXIT');
  FImages[3].LoadFromResourceName(HInstance,'OK');

  FMargin := (FImages[0].Height - FImages[2].Height) div 2;
  FRadius := FImages[2].Height div 2;
  FSliderRange := FImages[0].Width - 2*(FMargin + FRadius);
  FSliderPos := 0;
  FSliderDirection := sdLeft;

  FRoll := true;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  FSliderDirection := TSliderDirection((ord(FSliderDirection) + 1) mod 2);
  Timer1.Enabled := true;
end;

procedure TForm1.CheckBox1Change(Sender: TObject);
begin
  FRoll := Checkbox1.Checked;
end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  i: integer;
begin
  for i := 0 to High(FImages) do
    FImages[i].Free;
end;

procedure TForm1.FormPaint(Sender: TObject);
const
  TWO_PI = 2.0 * pi;
var
  x0, y0: Integer;
  imgEXIT, imgOK: TLazIntfImage;
  bmp: TBitmap;
  f: Double;
begin
  x0 := 20;
  y0 := 20;
  if FSliderPos > FSliderRange div 2 then
    Canvas.Draw(x0, y0, FImages[1])
  else
    Canvas.Draw(x0, y0, FImages[0]);

  imgExit := FImages[2].CreateIntfImage;
  imgOK := FImages[3].CreateIntfImage;
  bmp := TBitmap.Create;
  try
    f := FSliderPos / FSliderRange;
    BlendImages(imgOK, imgEXIT, 1.0 - f);
    if FRoll then
      RotateImage(imgOK, TWO_PI * f);

    bmp.Pixelformat := pf32Bit;
    bmp.Assign(imgOK);

    Canvas.Draw(x0 + FMargin + FSliderPos, y0 + FMargin, bmp);
  finally
    bmp.Free;
    imgOK.Free;
    imgExit.Free;
  end;
end;

procedure TForm1.SpinEdit1Change(Sender: TObject);
begin
  Timer1.Interval := SpinEdit1.Value;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  case FSliderDirection of
    sdRight:
      begin
        inc(FSliderPos, 2);
        if FSliderPos >= FSliderRange then
        begin
          FSliderPos := FSliderRange;
          Timer1.Enabled := false;
        end;
      end;
    sdLeft:
      begin
        dec(FSliderPos, 2);
        if FSliderPos <= 0 then
        begin
          FSliderPos := 0;
          Timer1.Enabled := false;
        end;
      end;
  end;
  Invalidate;
end;

end.

